import { from } from 'rxjs';
import { first, map, share } from 'rxjs/operators';
import * as PrettyCode from 'code-prettify-google';
import { CommonModule } from '@angular/common';
import { ErrorHandler, Injectable, Component, EventEmitter, Input, Output, ViewChild, NgModule, ViewChildren } from '@angular/core';
import { encode } from 'he';
import { MatSnackBar, MatButtonModule, MatCardModule, MatTabsModule } from '@angular/material';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class LoggerService {
    /**
     * @param {?} errorHandler
     */
    constructor(errorHandler) {
        this.errorHandler = errorHandler;
    }
    /**
     * @param {?} value
     * @param {...?} rest
     * @return {?}
     */
    log(value, ...rest) {
        console.log(value, ...rest);
    }
    /**
     * @param {?} error
     * @return {?}
     */
    error(error) {
        this.errorHandler.handleError(error);
    }
    /**
     * @param {?} value
     * @param {...?} rest
     * @return {?}
     */
    warn(value, ...rest) {
        console.warn(value, ...rest);
    }
}
LoggerService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
LoggerService.ctorParameters = () => [
    { type: ErrorHandler }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Wrapper around the prettify.js library
 */
class PrettyPrinterService {
    /**
     * @param {?} logger
     */
    constructor(logger) {
        this.logger = logger;
        // must to call and let the import function can be inject in to window
        // its a hack way
        console.log(PrettyCode);
        this.prettyPrintOne = from(this.getPrettyPrintOne()).pipe(share());
        // this.prettyPrintOne = of((window as any)['prettyPrintOne'])
    }
    /**
     * @private
     * @return {?}
     */
    getPrettyPrintOne() {
        /** @type {?} */
        const ppo = ((/** @type {?} */ (window)))['prettyPrintOne'];
        return Promise.resolve(ppo);
        // (window as any)['prettyPrintOne']
        // prettify.js is not in window global; load it with webpack loader
        // System.import('../script/prettify.js')
        //   .then(
        //     () => (window as any)['prettyPrintOne'],
        //     err => {
        //       const msg = `Cannot get prettify.js from server: ${err.message}`;
        //       this.logger.error(new Error(msg));
        //       // return a pretty print fn that always fails.
        //       return () => { throw new Error(msg); };
        //     });
    }
    /**
     * Format code snippet as HTML
     *  - false: don't display
     *  - true: do display
     *  - number: do display but start at the given number
     * @param {?} code
     * @param {?=} language
     * @param {?=} linenums
     * @return {?} Observable<string> - Observable of formatted code
     */
    formatCode(code, language, linenums) {
        return this.prettyPrintOne.pipe(map((/**
         * @param {?} ppo
         * @return {?}
         */
        ppo => {
            try {
                return ppo(code, language, linenums);
            }
            catch (err) {
                /** @type {?} */
                const msg = `Could not format code that begins '${code.substr(0, 50)}...'.`;
                console.error(msg, err);
                throw new Error(msg);
            }
        })), first());
    }
}
PrettyPrinterService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
PrettyPrinterService.ctorParameters = () => [
    { type: LoggerService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CopierService {
    /**
     * Creates a fake textarea element, sets its value from `text` property,
     * and makes a selection on it.
     * @param {?} text
     * @return {?}
     */
    createFake(text) {
        /** @type {?} */
        const docElem = (/** @type {?} */ (document.documentElement));
        /** @type {?} */
        const isRTL = docElem.getAttribute('dir') === 'rtl';
        // Create a fake element to hold the contents to copy
        this.fakeElem = document.createElement('textarea');
        // Prevent zooming on iOS
        this.fakeElem.style.fontSize = '12pt';
        // Reset box model
        this.fakeElem.style.border = '0';
        this.fakeElem.style.padding = '0';
        this.fakeElem.style.margin = '0';
        // Move element out of screen horizontally
        this.fakeElem.style.position = 'absolute';
        this.fakeElem.style[isRTL ? 'right' : 'left'] = '-9999px';
        // Move element to the same position vertically
        /** @type {?} */
        const yPosition = window.pageYOffset || docElem.scrollTop;
        this.fakeElem.style.top = yPosition + 'px';
        this.fakeElem.setAttribute('readonly', '');
        this.fakeElem.value = text;
        document.body.appendChild(this.fakeElem);
        this.fakeElem.select();
        this.fakeElem.setSelectionRange(0, this.fakeElem.value.length);
    }
    /**
     * @return {?}
     */
    removeFake() {
        if (this.fakeElem) {
            document.body.removeChild(this.fakeElem);
            this.fakeElem = null;
        }
    }
    /**
     * @param {?} text
     * @return {?}
     */
    copyText(text) {
        try {
            this.createFake(text);
            return document.execCommand('copy');
        }
        catch (err) {
            return false;
        }
        finally {
            this.removeFake();
        }
    }
}
CopierService.decorators = [
    { type: Injectable }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * If linenums is not set, this is the default maximum number of lines that
 * an example can display without line numbers.
 * @type {?}
 */
const DEFAULT_LINE_NUMS_COUNT = 10;
/**
 * Formatted Code Block
 *
 * Pretty renders a code block, used by code-example and code-tabs embedded components.
 *
 * Example usage:
 *
 * <app-code-core
 *   [language]="ts"
 *   [linenums]="true">
 * </app-code-core>
 *
 * Renders code provided through the `updateCode` method.
 */
class CoreComponent {
    /**
     * @param {?} snackbar
     * @param {?} pretty
     * @param {?} copier
     * @param {?} logger
     */
    constructor(snackbar, pretty, copier, logger) {
        this.snackbar = snackbar;
        this.pretty = pretty;
        this.copier = copier;
        this.logger = logger;
        this.codeFormatted = new EventEmitter();
    }
    /**
     * @return {?}
     */
    get code() {
        return this._code;
    }
    /**
     * @param {?} code
     * @return {?}
     */
    set code(code) {
        this._code = code;
        if (!this._code || !this.code.trim()) {
            this.showMissingCodeMessage();
        }
        else {
            this.formatDisplayedCode();
        }
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        //Called after the constructor, initializing input properties, and the first call to ngOnChanges.
        //Add 'implements OnInit' to the class.
    }
    /**
     * @return {?}
     */
    ngOnChanges() {
    }
    /**
     * Sets the message showing that the code could not be found.
     * @private
     * @return {?}
     */
    showMissingCodeMessage() {
        this.setCodeHtml(`<p class="code-missing">The code sample is missing! </p>`);
    }
    /**
     * @private
     * @return {?}
     */
    formatDisplayedCode() {
        /** @type {?} */
        const leftAlignedCode = this.leftAlign(this.code);
        this.setCodeHtml(leftAlignedCode); // start with unformatted code
        this.codeText = this.getCodeText(); // store the unformatted code as text (for copying)
        this.pretty
            .formatCode(leftAlignedCode, this.language, this.getLinenums(leftAlignedCode))
            .subscribe((/**
         * @param {?} c
         * @return {?}
         */
        c => { return this.setCodeHtml(c); }), (/**
         * @param {?} err
         * @return {?}
         */
        err => { }));
    }
    /**
     * let the code string left indent until one of the code line touch the left side without space
     * @private
     * @param {?} text
     * @return {?}
     */
    leftAlign(text) {
        /** @type {?} */
        let indent = Number.MAX_VALUE;
        /** @type {?} */
        const lines = text.split('\n');
        lines.forEach((/**
         * @param {?} line
         * @return {?}
         */
        line => {
            /** @type {?} */
            const lineIndent = line.search(/\S/);
            if (lineIndent !== -1) {
                indent = Math.min(lineIndent, indent);
            }
        }));
        return lines.map((/**
         * @param {?} line
         * @return {?}
         */
        line => line.substr(indent))).join('\n').trim();
    }
    /**
     * Sets the innerHTML of the code container to the provided code string.
     * @private
     * @param {?} formattedCode
     * @return {?}
     */
    setCodeHtml(formattedCode) {
        // **Security:** Code example content is provided by docs authors and as such its considered to
        // be safe for innerHTML purposes.
        this.codeContainer.nativeElement.innerHTML = formattedCode;
    }
    /**
     * Gets the textContent of the displayed code element.
     * @private
     * @return {?}
     */
    getCodeText() {
        // `prettify` may remove newlines, e.g. when `linenums` are on. Retrieve the content of the
        // container as text, before prettifying it.
        // We take the textContent because we don't want it to be HTML encoded.
        return this.codeContainer.nativeElement.textContent;
    }
    /**
     * Copies the code snippet to the user's clipboard.
     * @return {?}
     */
    doCopy() {
        /** @type {?} */
        const code = this.codeText;
        /** @type {?} */
        const successfullyCopied = this.copier.copyText(code);
        if (successfullyCopied) {
            this.logger.log('Copied code to clipboard:', code);
            this.snackbar.open('Code Copied', '', { duration: 800 });
        }
        else {
            this.logger.error(new Error(`ERROR copying code to clipboard: "${code}"`));
            this.snackbar.open('Copy failed. Please try again!', '', { duration: 800 });
        }
    }
    /**
     * Gets the calculated value of linenums (boolean/number).
     * @param {?} code
     * @return {?}
     */
    getLinenums(code) {
        /** @type {?} */
        const linenums = typeof this.linenums === 'boolean' ? this.linenums :
            this.linenums === 'true' ? true :
                this.linenums === 'false' ? false :
                    typeof this.linenums === 'string' ? parseInt(this.linenums, 10) :
                        this.linenums;
        // if no linenums, enable line numbers if more than one line
        return linenums == null || isNaN((/** @type {?} */ (linenums))) ?
            (code.match(/\n/g) || []).length > DEFAULT_LINE_NUMS_COUNT : linenums;
    }
    /**
     * @return {?}
     */
    open() {
        this.snackbar.open('Code Copied', '', { duration: 800 });
    }
}
CoreComponent.decorators = [
    { type: Component, args: [{
                selector: 'app-code-core',
                template: "<pre class=\"prettyprint lang-{{language}}\">\n  <button\n    class=\"material-icons copy-button\"\n    title=\"Copy code snippet\"\n    (click)=\"doCopy()\">\n    <span aria-hidden=\"true\">c</span>\n  </button>\n  <code class=\"animated fadeIn\" #codeContainer></code>\n</pre>\n\n",
                styles: [".prettyprint{position:relative}.copy-button{position:absolute;top:-7px;right:-19px;padding:0;color:#b0bec5;background-color:transparent;border:none;cursor:pointer}.copy-button:hover{color:#6e6e6e}ol.linenums /deep/{margin:0;font-family:Roboto,\"Helvetica Neue Light\",\"Helvetica Neue\",Helvetica,Arial,\"Lucida Grande\",sans-serif;color:#b3b6b7}ol.linenums /deep/ li{margin:0;font-family:\"Droid Sans Mono\",monospace;font-size:90%;line-height:24px}pre{display:flex;min-height:2rem;margin:1rem 1.5rem;white-space:pre-wrap;align-items:center}pre code span{line-height:1.5rem}pre.lang-bash,pre.lang-sh{background-color:#333}pre.lang-bash /deep/ .atn,pre.lang-bash /deep/ .atv,pre.lang-bash /deep/ .blk,pre.lang-bash /deep/ .com,pre.lang-bash /deep/ .dec,pre.lang-bash /deep/ .kwd,pre.lang-bash /deep/ .lit,pre.lang-bash /deep/ .otl,pre.lang-bash /deep/ .pln,pre.lang-bash /deep/ .pnk,pre.lang-bash /deep/ .pun,pre.lang-bash /deep/ .str,pre.lang-bash /deep/ .tag,pre.lang-bash /deep/ .typ,pre.lang-sh /deep/ .atn,pre.lang-sh /deep/ .atv,pre.lang-sh /deep/ .blk,pre.lang-sh /deep/ .com,pre.lang-sh /deep/ .dec,pre.lang-sh /deep/ .kwd,pre.lang-sh /deep/ .lit,pre.lang-sh /deep/ .otl,pre.lang-sh /deep/ .pln,pre.lang-sh /deep/ .pnk,pre.lang-sh /deep/ .pun,pre.lang-sh /deep/ .str,pre.lang-sh /deep/ .tag,pre.lang-sh /deep/ .typ{color:#17ff0b}pre /deep/ .pln{color:#000}@media screen{pre /deep/ .str{color:#800}pre /deep/ .kwd{color:#00f}pre /deep/ .com{color:#060}pre /deep/ .typ{color:red}pre /deep/ .lit{color:#08c}pre /deep/ .clo,pre /deep/ .opn,pre /deep/ .pun{color:#660}pre /deep/ .tag{color:#008}pre /deep/ .atn{color:#606}pre /deep/ .atv{color:#800}pre /deep/ .dec,pre /deep/ .var{color:#606}pre /deep/ .fun{color:red}}@media print,projection{pre /deep/ .str{color:#060}pre /deep/ .kwd{color:#006;font-weight:700}pre /deep/ .com{color:#600;font-style:italic}pre /deep/ .typ{color:#404;font-weight:700}pre /deep/ .lit{color:#044}pre /deep/ .clo,pre /deep/ .opn,pre /deep/ .pun{color:#440}pre /deep/ .tag{color:#006;font-weight:700}pre /deep/ .atn{color:#404}pre /deep/ .atv{color:#060}}"]
            }] }
];
/** @nocollapse */
CoreComponent.ctorParameters = () => [
    { type: MatSnackBar },
    { type: PrettyPrinterService },
    { type: CopierService },
    { type: LoggerService }
];
CoreComponent.propDecorators = {
    language: [{ type: Input }],
    linenums: [{ type: Input }],
    codeFormatted: [{ type: Output }],
    codeContainer: [{ type: ViewChild, args: ['codeContainer',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CoreModule {
}
CoreModule.decorators = [
    { type: NgModule, args: [{
                declarations: [CoreComponent],
                imports: [
                    CommonModule,
                    MatButtonModule
                ],
                exports: [
                    CoreComponent
                ],
                providers: [
                    PrettyPrinterService,
                    CopierService,
                    LoggerService,
                ],
                entryComponents: [CoreComponent],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * An embeddable code block that displays nicely formatted code.
 *
 * Example usage:
 *
 * by pass the code block by params
 * Example usage:
 *
 * <code-example [config]="config">
 * </code-example>
 *
 * If both modes exist simultaneously, use the second one.
 */
class CodeExampleComponent {
    constructor() { }
    /**
     * @return {?}
     */
    ngOnInit() {
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        // parse the encode code
        /** @type {?} */
        const encodeHTML = encode(this.config.code, { 'encodeEverything': true });
        this.coreContent.code = encodeHTML;
    }
}
CodeExampleComponent.decorators = [
    { type: Component, args: [{
                selector: 'app-code-example',
                template: "\n<div class=\"code-content\">\n  <!-- For SnackBar Lazy loading https://github.com/akveo/nebular/issues/943 -->\n  <div style=\"display: none;\">\n    <ng-content></ng-content>\n  </div>\n \n  <app-code-header *ngIf=\"config.header\" [header]=\"config.header\"></app-code-header>\n\n  <app-code-core [language]=\"config.language\" [linenums]=\"config.linenums\">\n  </app-code-core>\n</div>\n",
                styles: [".code-content{background-color:rgba(241,241,241,.2);border:.5px solid #dbdbdb;border-radius:5px;color:#333;margin:16px auto}"]
            }] }
];
/** @nocollapse */
CodeExampleComponent.ctorParameters = () => [];
CodeExampleComponent.propDecorators = {
    config: [{ type: Input }],
    content: [{ type: ViewChild, args: ['content',] }],
    coreContent: [{ type: ViewChild, args: [CoreComponent,] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class HeaderComponent {
    constructor() { }
    /**
     * @return {?}
     */
    ngOnInit() {
    }
}
HeaderComponent.decorators = [
    { type: Component, args: [{
                selector: 'app-code-header',
                template: "<header>\n  {{ header }}\n</header>",
                styles: ["header{background-color:#1e88e5;border-radius:5px 5px 0 0;color:#fafafa;font-size:16px;padding:8px 16px}"]
            }] }
];
/** @nocollapse */
HeaderComponent.ctorParameters = () => [];
HeaderComponent.propDecorators = {
    header: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CodeExampleModule {
}
CodeExampleModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    CodeExampleComponent,
                    HeaderComponent
                ],
                imports: [
                    CommonModule,
                    CoreModule
                ],
                exports: [
                    CodeExampleComponent
                ],
                entryComponents: [CodeExampleComponent],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Renders a set of tab group of code snippets.
 *
 * The innerHTML of the `<code-tabs>` component should contain `<code-pane>` elements.
 * Each `<code-pane>` has the same interface as the embedded `<code-example>` component.
 * The optional `linenums` attribute is the default `linenums` for each code pane.
 */
class CodeTabsComponent {
    /**
     * @return {?}
     */
    ngOnInit() {
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        this.coreComponents.toArray().forEach((/**
         * @param {?} codeComponent
         * @param {?} i
         * @return {?}
         */
        (codeComponent, i) => {
            codeComponent.code = encode(this.configs[i].code, { 'encodeEverything': true });
        }));
    }
}
CodeTabsComponent.decorators = [
    { type: Component, args: [{
                selector: 'app-code-tabs',
                template: "<!-- Use content projection so that the provided HTML's code-panes can be split into tabs -->\n<mat-card>\n  <mat-tab-group class=\"code-tab-group\" disableRipple>\n    <mat-tab style=\"overflow-y: hidden;\" *ngFor=\"let config of configs\">\n      <ng-template mat-tab-label>\n        <span>{{ config.header }}</span>\n      </ng-template>\n      <app-code-core\n        [language]=\"config.language\"\n        [linenums]=\"config.linenums\">\n      </app-code-core>\n    </mat-tab>\n  </mat-tab-group>\n</mat-card>",
                styles: ["mat-card /deep/ .mat-tab-body-content{height:auto}mat-card /deep/ .mat-tab-body-wrapper mat-tab-body .mat-tab-body{overflow-y:hidden;background:red}"]
            }] }
];
CodeTabsComponent.propDecorators = {
    configs: [{ type: Input }],
    linenums: [{ type: Input }],
    content: [{ type: ViewChild, args: ['content',] }],
    coreComponents: [{ type: ViewChildren, args: [CoreComponent,] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CodeTabsModule {
}
CodeTabsModule.decorators = [
    { type: NgModule, args: [{
                declarations: [CodeTabsComponent],
                imports: [
                    CommonModule,
                    MatCardModule,
                    MatTabsModule,
                    CoreModule
                ],
                exports: [
                    CodeTabsComponent
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class PrettyCodeModule {
}
PrettyCodeModule.decorators = [
    { type: NgModule, args: [{
                declarations: [],
                imports: [
                    CommonModule
                ],
                exports: [
                    CoreModule,
                    CodeExampleModule,
                    CodeTabsModule,
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { PrettyCodeModule, HeaderComponent, CodeExampleComponent as ɵg, CodeExampleModule as ɵf, CodeTabsComponent as ɵi, CodeTabsModule as ɵh, CoreComponent as ɵb, CoreModule as ɵa, CopierService as ɵe, LoggerService as ɵd, PrettyPrinterService as ɵc };

//# sourceMappingURL=eriice-pretty-code.js.map